@@ -5,7 +5,7 @@
 # -- partly taken from PureFTPd
 #
 
-VERSION=1.2.8
+VERSION=1.5.0
 
 
 # exits with a custom error message
@@ -39,7 +39,7 @@ This script is meant to make installing SAMHAIN as easy as
 possible.  Just read the text below, hit ENTER, and you're
 on your way. 
 
-SAMHAIN ships with no warranty whatsoever, without
+SAMHAIN ships with NO WARRANTY whatsoever, without
 even the implied warranty of merchantability or fitness 
 for a particular purpose. The author takes no responsibility
 for the consequences of running this script.
@@ -112,9 +112,10 @@ $dialog \
 --separate-output \
 --backtitle "Samhain $VERSION" \
 --radiolist "Samhain can run as standalone application on a single dektop machine, or as a client/server application for centralized monitoring of many hosts" \
-10 75 2 \
-"disable-network"   "Single desktop machine" on \
-"enable-network"    "Network (client/server)" off \
+10 75 3 \
+"disable-network"          "Single desktop machine" on \
+"enable-network=client"    "Network (client)" off \
+"enable-network=server"    "Network (server)" off \
 2> $tmp
 
 mtest=$?
@@ -128,6 +129,7 @@ else
 fi
 
 cfgtest=`echo $cfgline | grep disable`
+# srvtest=`echo $cfgline | grep server`
 #------------------------------------------------------------
 #
 #  Server options
@@ -148,7 +150,7 @@ fi
 
 
 $dialog \
---title 'Server options' \
+--title 'Network options' \
 --separate-output \
 --backtitle "Samhain $VERSION" \
 --checklist 'Use SPACE to set/unset. If in doubt, read the manual.' \
@@ -167,7 +169,7 @@ if [ $mtest = 0 ]; then
 fi
 
 $dialog \
---title 'Server options' \
+--title 'Network options' \
 --backtitle "Samhain $VERSION" \
 --inputbox "Server port" 10 75 "49777" \
 2> $tmp
@@ -186,7 +188,7 @@ fi
 
 
 $dialog \
---title 'Server options' \
+--title 'Network options' \
 --backtitle "Samhain $VERSION" \
 --inputbox "Server address" 10 75 "127.0.0.1" \
 2> $tmp
@@ -203,7 +205,7 @@ if [ $mtest = 0 ]; then
 fi
 
 $dialog \
---title "Server options" \
+--title "Network options" \
 --backtitle 'Samhain $VERSION' \
 --inputbox "Backup server address" 10 75 "none" \
 2> $tmp
@@ -249,7 +251,7 @@ $dialog \
 --checklist 'Use SPACE to set/unset. If in doubt, read the MANUAL.' \
 20 75 10 \
 'enable-static' "Don't link with shared libraries" on \
-'with-suidcheck' "Check for suid/sgid files" on \
+'enable-suidcheck' "Check for suid/sgid files" on \
 'enable-login-watch' "Watch for login/logout events" off \
 'enable-ptrace' "Enable anti-debugger code" off \
 'enable-db-reload' "Reload database on SIGHUP" off \
@@ -362,23 +364,6 @@ if [ $mtest = 0 ]; then
 fi
   
 
-$dialog \
---title "Signed database and configuration" \
---backtitle "Samhain $VERSION" \
---inputbox "Please enter the TIGER192 checksum of the gpg binary as output by \"gpg --print-md TIGER192 /path/to/gpg\"" \
-10 75 "/usr/bin/gpg: D9701E46 62155951 FA2944E8  78572062 C8839FE3 1A0734BB" \
-2> $tmp
-
-mtest=$?
-
-if [ $mtest = -1 ]; then
-    exit 1
-fi
-if [ $mtest = 0 ]; then
-    z=`cat $tmp`
-    cfgline2=$z
-fi
-
 fi  
 # want signed
 fi
@@ -452,9 +437,9 @@ else
 fi
 
 if [ "x$FULL_STEALTH" = "xyes" ]; then
-    cfgline="$cfgline --with-stealth=$mnum"
+    cfgline="$cfgline --enable-stealth=$mnum"
 else
-    cfgline="$cfgline --with-micro-stealth=$mnum"
+    cfgline="$cfgline --enable-micro-stealth=$mnum"
 fi
 
 # if [ "x$mtest" != "xnone" ]; then
@@ -475,7 +460,7 @@ fi
 if [ $mtest = 0 ]; then
     mfile=`cat $tmp`
     for z in $mfile ; do
-	cfgline="$cfgline --with-install-name=$z"
+	cfgline="$cfgline --enable-install-name=$z"
     done
 fi
 
@@ -483,7 +468,7 @@ $dialog \
 --title "Stealth options" \
 --separate-output \
 --backtitle "Samhain $VERSION" \
---inputbox "You can set a magic string such that the command line will be ignored unless the first argument is this magic string. If you do not want this, select CANCEL, otherwise choose a string and select OK." \
+--inputbox "You can set a magic string such that command line arguments will be ignored unless the first argument is this magic string, and read from stdin otherwise. If you do not want this, select CANCEL, otherwise choose a string and select OK." \
 10 75 "foo" \
 2> $tmp
 
@@ -494,7 +479,7 @@ fi
 if [ $mtest = 0 ]; then
     mfile=`cat $tmp`
     for z in $mfile ; do
-	cfgline="$cfgline --with-nocl=$z"
+	cfgline="$cfgline --enable-nocl=$z"
     done
 fi
 
@@ -512,7 +497,7 @@ if [ $mtest = -1 ]; then
     exit 1
 fi
 if [ $mtest = 0 ]; then
-    cfgline="$cfgline --with-khide"
+    cfgline="$cfgline --enable-khide"
 fi
 
 # f [ "x$I386_LINUX" = "xyes" ]; then
@@ -530,7 +515,7 @@ $dialog \
 --title 'Paths' \
 --separate-output \
 --backtitle "Samhain $VERSION" \
---radiolist "Do you wish to change the default paths ?\n\nThe default paths are:\n\n /usr/local/sbin all binaries\n /etc/samhainrc configuration file\n /var/lib/samhain/samhain_file data file\n /var/log/samhain_log log file\n /var/run/samhain.pid lock file $HTML" 20 76 5 \
+--radiolist "Do you wish to change the default paths ?\n\nThe default paths are:\n\n /usr/local/sbin all binaries\n /etc/samhainrc configuration file\n /var/lib/samhain/samhain_file data file\n /var/log/samhain_log log file\n /var/run/samhain.pid pid file $HTML" 20 76 5 \
 'usr'    "Install binaries in /usr/sbin" off \
 'opt'    "Use /opt/samhain, /etc/opt, /var/opt" off \
 'all'    "Set paths individually" off \
@@ -548,10 +533,10 @@ if [ $mtest = 0 ]; then
 mfile=`cat $tmp`
 for z in $mfile ; do
     if [ "x$z" = "xopt" ]; then
-	    cfgline="$cfgline --prefix=/opt"
+	    cfgline="$cfgline --prefix=OPT"
     fi
     if [ "x$z" = "xusr" ]; then
-	    cfgline="$cfgline --prefix=/usr"
+	    cfgline="$cfgline --prefix=USR"
     fi
     if [ "x$z" = "xall" ]; then
 $dialog \
@@ -653,7 +638,7 @@ fi
 if [ $mtest = 0 ]; then
     mfile=`cat $tmp`
     for z in $mfile ; do
-	cfgline="$cfgline --with-lock-file=$z"
+	cfgline="$cfgline --with-pid-file=$z"
     done
 fi
 
