@@ -1050,6 +1050,9 @@ int main(int argc, char * argv[])
 	}
     }
   
+  /* if fd 0 is closed, presume that we want to be daemon and
+   * run in check mode
+   */
   if ((-1) == retry_fcntl(FIL__, __LINE__, 0, F_GETFL, 0) && 
 	   errno == EBADF)
     {
@@ -1171,6 +1174,7 @@ int main(int argc, char * argv[])
 	  my_argv[0] = argv[0]; ++my_argc;  
 	  command_line[0] = '\0';
 	  fgets (command_line, 255, stdin);
+	  command_line[255] = '\0';
 	  do {
 	    my_argv[my_argc] = 
 	      strtok( (my_argc == 1) ? command_line : NULL, " \n"); 
@@ -1235,10 +1239,11 @@ int main(int argc, char * argv[])
    */
   if (sh.flag.checkSum == SH_CHECK_INIT && sl_is_suid()) 
     {
+      dlog(1, FIL__, __LINE__, 
+	   _("Cannot initialize database when running with SUID credentials.\nYou need to run this with the user ID %d.\nYour current user ID is %d."), 
+	   (int) sl_ret_euid(), (int) sh.real.uid);
       sh_error_handle ((-1), FIL__, __LINE__, EACCES, MSG_ACCESS,
 		       (long) sh.real.uid, sh.data.path);
-      sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EXIT_ABORT1,
-		       sh.prg_name);
       aud_exit(FIL__, __LINE__, EXIT_FAILURE);
     }
 
@@ -1250,12 +1255,7 @@ int main(int argc, char * argv[])
   /* initialize signal handling etc
    */
   if (sh.flag.isdaemon == S_TRUE)
-    { 
-      sh_error_only_stderr (BAD);
-#if defined(WITH_TRACE) || defined(WITH_TPT) 
-      dbg_use_console();
-#endif
-    }
+    sh_error_only_stderr (BAD);
 
   if (sh_unix_init(sh.flag.isdaemon) == -1) 
     {
