@@ -34,6 +34,8 @@
 #include "samhain.h"
 #include "sh_error.h"
 
+char aud_err_message[64];
+
 typedef struct cht_struct 
 {
   char           * str;
@@ -222,7 +224,8 @@ long int retry_fstat(char * file, int line, int filed, struct stat *buf)
   } while (val_retry < 0 && errno == EINTR);
   error = errno;
   if (val_retry < 0) {
-      sh_error_handle ((-1), file, line, error, MSG_ERR_FSTAT, sh_error_message(error),
+      sh_error_handle ((-1), file, line, error, MSG_ERR_FSTAT, 
+		       sh_error_message(error),
 		       (long) filed );
   }
   errno = error;    
@@ -442,15 +445,18 @@ long int aud_open (char * file, int line, int privs,
   if (privs == SL_YESPRIV)
     sl_unset_suid();
   
+  if (val_return < 0)
+    sl_strlcpy(aud_err_message, sh_error_message(error), 64);
+
   if (sh.flag.audit && (sh.flag.aud_mask & AUD_OPEN) != 0)
     {
       sh_error_handle ((-1), file, line, 0, MSG_AUD_OPEN,
 		       pathname, (long) flags, (long) mode, val_return);
     }
   if (val_return < 0) {
-      sh_error_handle ((-1), file, line, error, MSG_ERR_OPEN, 
-		       sh_error_message(error),
-		       pathname, (long) flags, (long) mode, val_return);
+    sh_error_handle ((-1), file, line, error, MSG_ERR_OPEN, 
+		     sh_error_message(error),
+		     pathname, (long) flags, (long) mode, val_return);
   }
   errno = error;
   SL_RETURN(val_return, _("aud_open"));
