@@ -124,6 +124,7 @@ enum {
  MSG_FI_FAIL,     
  MSG_FI_GLOB,
  MSG_FI_COLL,
+ MSG_FI_DOUBLE,
  MSG_FI_2LONG,    
  MSG_FI_2LONG2,   
  MSG_FI_NOPATH,   
@@ -171,9 +172,12 @@ enum {
  MSG_TCP_CREG,    
  MSG_TCP_FAUTH,   
  MSG_TCP_TIMOUT,  
- MSG_TCP_LOOKERR,
+
+ MSG_TCP_RESCLT,
+ MSG_TCP_RESPEER,
  MSG_TCP_LOOKERS,
  MSG_TCP_LOOKUP,
+
  MSG_TCP_TIMEXC,  
  MSG_TCP_NOCLT,   
  MSG_TCP_BADCONN, 
