@@ -56,7 +56,7 @@ typedef struct dbins_ {
   char            time[20];
   char            msg[1024];
   char            sev[8];
-  char            path[1024];
+  char            path[12288];
   char            user[9];
   char            group[9];
   char            program[8];
@@ -124,7 +124,7 @@ static my_attr attr_tab[] = {
   { NULL, N_("remote_host"), 0,   3,   64, offsetof(struct dbins_, host) },
   { NULL, N_("msg"),         0,   4, 1024, offsetof(struct dbins_, msg) },
 
-  { NULL, N_("path"),        0,   5, 1024, offsetof(struct dbins_, path)  },
+  { NULL, N_("path"),        0,   5,12288, offsetof(struct dbins_, path)  },
   { NULL, N_("username"),    0,   6,    9, offsetof(struct dbins_, user)  },
   { NULL, N_("group"),       0,   7,    9, offsetof(struct dbins_, group)  },
   { NULL, N_("program"),     0,   8,    8, offsetof(struct dbins_, program)  },
@@ -468,13 +468,15 @@ char * null_or_val (char * end,   char * val,   int * size, int flag)
   return end;
 }
 
+#define SH_QUERY_MAX 16383
+
 long sh_database_entry (dbins * db_entry, long id)
 {
   /* This does not need to be re-entrant
    */
-  static char   query[2048];
+  char * query;
   static char   columns[1024];
-  static char   values[1024];
+  char * values;
 
   int    status;
   long   the_id;
@@ -493,6 +495,9 @@ long sh_database_entry (dbins * db_entry, long id)
 
   SL_ENTER(_("sh_database_entry"));
 
+  query  = SH_ALLOC(SH_QUERY_MAX+1);
+  values = SH_ALLOC(SH_QUERY_MAX+1);
+
   md5Reset(&crc);
 
   if (db_entry->host[0] == '\0')
@@ -507,7 +512,7 @@ long sh_database_entry (dbins * db_entry, long id)
     sprintf(num, "%ld", id);
 
 
-  sl_snprintf (values, 1023, _("(%s,%c%s%c,%c%s%c,%c%s%c,%c%s%c"),
+  sl_snprintf (values, SH_QUERY_MAX, _("(%s,%c%s%c,%c%s%c,%c%s%c,%c%s%c"),
 	       id >= 0 ? num : _("NULL"),
 	       '\'', db_entry->host,'\'', 
 	       '\'', db_entry->time,'\'', 
@@ -529,7 +534,7 @@ long sh_database_entry (dbins * db_entry, long id)
     md5Update(&crc, db_entry->msg,  strlen(db_entry->sev));
 
 
-  size   =    1023 - strlen(values);
+  size   =  SH_QUERY_MAX - strlen(values);
   end    =  values + strlen(values);
   c_size =    1023 - strlen(columns);
   c_end  = columns + strlen(columns);
@@ -596,13 +601,16 @@ long sh_database_entry (dbins * db_entry, long id)
   if (db_table[0]    == '\0')
     sl_strlcpy(db_table, _("log"),       64);
 
-  sl_snprintf (query, 2047,
+  sl_snprintf (query, SH_QUERY_MAX,
 	       _("INSERT INTO %s %s VALUES %s"),
 	       db_table, columns, values);
 
   /* fprintf(stderr, "%s\n", query); */ 
   status = sh_database_query (query, &the_id);
 
+  SH_FREE(query);
+  SH_FREE(values);
+  
   SL_RETURN(the_id, _("sh_database_entry"));
 }
 
