@@ -253,16 +253,23 @@ int  sh_log_console (char *errmsg)
   static int logkey_seen = 0;
   int    error;
   struct sigaction sa_new, sa_old;
+  static int blockMe = 0;
 
   SL_ENTER(_("sh_log_console"));
 
-  ASSERT_RET((errmsg != NULL), _("errmsg != NULL"), 0);
+  if (errmsg == 0 || blockMe == 1)
+    {
+      SL_RETURN(0, _("sh_log_console"));
+    }
+  else
+    blockMe = 1;
 
   /* Don't print the LOGKEY to the console
    */
   if (logkey_seen == 0 && NULL != sl_strstr(errmsg, _("LOGKEY")))
     { 
       logkey_seen = 1;
+      blockMe = 0;
       SL_RETURN(0, _("sh_log_console"));
     }
 
@@ -281,10 +288,11 @@ int  sh_log_console (char *errmsg)
    */
   retry_sigaction(FIL__, __LINE__, SIGPIPE, &sa_new, &sa_old);
  
-  if (sh.flag.isdaemon == S_FALSE)
+  if (sh.flag.isdaemon == S_FALSE || OnlyStderr == S_TRUE)
     {
       fprintf (stderr, "%s\n", errmsg);
       retry_sigaction(FIL__, __LINE__, SIGPIPE, &sa_old, NULL);
+      blockMe = 0;
       SL_RETURN(0, _("sh_log_console"));
     }
 
@@ -295,7 +303,8 @@ int  sh_log_console (char *errmsg)
       fd[0] = open ( sh.srvcons.name, O_WRONLY|O_APPEND|O_NOCTTY|O_NONBLOCK);
       if (fd[0] >= 0) {
 	sflags = retry_fcntl(FIL__, __LINE__, fd[0], F_GETFL, 0);
-	retry_fcntl(FIL__, __LINE__, fd[0], F_SETFL, sflags & ~O_NONBLOCK);
+	if (sflags >= 0)
+	  retry_fcntl(FIL__, __LINE__, fd[0], F_SETFL, sflags & ~O_NONBLOCK);
       }
 
       if (sh.srvcons.alt != NULL && sh.srvcons.alt[0] != '\0')
@@ -303,7 +312,8 @@ int  sh_log_console (char *errmsg)
 	  fd[1] = open (sh.srvcons.alt, O_WRONLY|O_APPEND|O_NOCTTY|O_NONBLOCK);
 	  if (fd[1] >= 0) {
 	    sflags = retry_fcntl(FIL__, __LINE__, fd[1], F_GETFL, 0);
-	    retry_fcntl(FIL__, __LINE__, fd[1], F_SETFL, sflags & ~O_NONBLOCK);
+	    if (sflags >= 0)
+	      retry_fcntl(FIL__, __LINE__,fd[1],F_SETFL, sflags & ~O_NONBLOCK);
 	    ccMax = 2;
 	  }
 	}
@@ -349,6 +359,7 @@ int  sh_log_console (char *errmsg)
     retval = 0;
 
   retry_sigaction(FIL__, __LINE__, SIGPIPE, &sa_old, NULL);
+  blockMe = 0;
   SL_RETURN(retval, _("sh_log_console"));
 }
 
