@@ -35,6 +35,9 @@
 #undef  FIL__
 #define FIL__  _("sh_err_log.c")
 
+#undef  FIX_XML
+#define FIX_XML 1 
+
 #define MYSIGLEN (2*KEY_LEN + 32)
 
 typedef struct _sh_log_buf {
@@ -133,6 +136,7 @@ int sh_error_logverify (char * s)
   char * bufc;
 #ifdef SH_USE_XML
   char * ptr;
+  int fixed_xml = S_TRUE;
 #endif
   char signature[64];
   char key[KEY_LEN+1];
@@ -140,6 +144,7 @@ int sh_error_logverify (char * s)
   char timestamp[64];
   char c_cont;
   int  chk_mode = CHK_KEY;
+  char c_start;
 
   sh_error_logoff();
 
@@ -165,6 +170,11 @@ int sh_error_logverify (char * s)
   c_cont ^= XOR_CODE;
 #endif
 
+  c_start  = '<';
+#ifdef SH_STEALTH
+  c_start ^= XOR_CODE;
+#endif
+
   buf  = (char *) SH_ALLOC( 2*SH_BUFSIZE+1 );
   bufc = (char *) SH_ALLOC( 2*SH_BUFSIZE+1 );
 
@@ -293,15 +303,34 @@ int sh_error_logverify (char * s)
 	       * A48014C05604EF7C9472330E85453E704024943E556163C2
 	       */
 #ifdef SH_USE_XML
-	      sl_strlcpy(signature, &bufc[4], KEY_LEN+1);
-	      sl_strlcpy(signature, &bufc[4], KEY_LEN+1);
+#ifdef SH_STEALTH
+	      if (bufc[0] == c_start) /* FIX XML */
+#else
+	      if (bufc[0] == c_start)
+#endif
+		{
+		  sl_strlcpy(signature, &bufc[5], KEY_LEN+1);
+		  fixed_xml = S_TRUE;
+		}
+	      else
+		{
+		  sl_strlcpy(signature, &bufc[4], KEY_LEN+1);
+		  fixed_xml = S_FALSE;
+		}
 	      if (sl_strlen(bufc) > (KEY_LEN+18))
 		{
-		  sl_strlcpy(timestamp, &bufc[KEY_LEN+4], 64);
 #ifdef SH_STEALTH
-		  ptr = strchr(timestamp, '<' ^ XOR_CODE);
+		  if (bufc[0] == c_start) /* FIX XML */
 #else
-		  ptr = strchr(timestamp, '<');
+		  if (bufc[0] == c_start)
+#endif
+		    sl_strlcpy(timestamp, &bufc[KEY_LEN+5], 64);
+		  else
+		    sl_strlcpy(timestamp, &bufc[KEY_LEN+4], 64);
+#ifdef SH_STEALTH
+		  ptr = strchr(timestamp, c_start);
+#else
+		  ptr = strchr(timestamp, c_start);
 #endif
 		  if (ptr) *ptr = '\0';
 		}
@@ -408,9 +437,23 @@ int sh_error_logverify (char * s)
 	{
 #ifdef SH_USE_XML
 	  if (chk_mode == CHK_NON)
-	    fprintf (stdout, _("XFAIL: line=%05d %s/log>\n"), count-1, buf);
+	    {
+	      if (fixed_xml == S_FALSE)
+		fprintf (stdout, _("XFAIL: line=%05d %s/log>\n"), 
+			 count-1, buf);
+	      else
+		fprintf (stdout, _("XFAIL: line=%05d %s</log>\n"), 
+			 count-1, buf);
+	    }
 	  else
-	    fprintf (stdout, _("FAIL:  line=%05d %s/log>\n"), count-1, buf);
+	    {
+	      if (fixed_xml == S_FALSE)
+		fprintf (stdout, _("FAIL:  line=%05d %s/log>\n"), 
+			 count-1, buf);
+	      else
+		fprintf (stdout, _("FAIL:  line=%05d %s</log>\n"), 
+			 count-1, buf);
+	    }
 #else
 	  if (chk_mode == CHK_NON)
 	    fprintf (stdout, _("XFAIL: line=%5d %s\n"), count-1, buf);
@@ -421,7 +464,10 @@ int sh_error_logverify (char * s)
       else
 	{
 #ifdef SH_USE_XML 
-	  fprintf (stdout, _("PASS:  line=%05d %s/log>\n"), count-1, buf);
+	  if (fixed_xml == S_FALSE)
+	    fprintf (stdout, _("PASS:  line=%05d %s/log>\n"),  count-1, buf);
+	  else
+	    fprintf (stdout, _("PASS:  line=%05d %s</log>\n"), count-1, buf);
 #else
 	  fprintf (stdout, _("PASS:  line=%5d %s\n"), count-1, buf);
 #endif    
@@ -604,22 +650,35 @@ int  sh_log_file (char *errmsg)
    */
   sl_strlcpy (log_msg.msg, errmsg, status+1 );
 
+
 #ifdef SH_USE_XML
   /* cut the trailing "/>"
    */
   if (log_msg.msg[status-2] == '/')
     {
+#ifdef FIX_XML
+      log_msg.msg[status-2] = ' '; /* ' ' FIX XML */
+      log_msg.msg[status-1] = '>'; /* '>' FIX XML */
+#else
       log_msg.msg[status-2] = '>'; /* ' ' FIX XML */
       log_msg.msg[status-1] = '<'; /* '>' FIX XML */
+#endif
       log_msg.msg[status]   = '\0';
     }
-  else if (log_msg.msg[status-5] == '/')
+  else if (status >= 6 && log_msg.msg[status-5] == '/' && 
+	   log_msg.msg[status-6] == '<')
     {
+#ifdef FIX_XML
+      log_msg.msg[status-6]   = '\0';
+      status -= 6;
+#else
       log_msg.msg[status-5]   = '\0';
       status -= 5;
+#endif
     }
 #endif
 
+
 #ifdef SH_STEALTH
   sh_do_encode (log_msg.msg, status);
 #endif
@@ -679,6 +738,7 @@ int  sh_log_file (char *errmsg)
 
   /* --- Sign the message with the signature key. ---
    */
+
   sl_strlcat (log_msg.msg, skey->sigkey_new, status + KEY_LEN + 2);
   
   sl_strlcpy (log_msg.signature,
@@ -689,11 +749,19 @@ int  sh_log_file (char *errmsg)
 #ifdef SH_USE_XML
   if (log_msg.timestamp[0] != '\0')
     sprintf(log_msg.sig,                            /* known to fit  */
+#ifdef FIX_XML
+	    _("\n<sig>%s%s</sig></log>\n"),          /* <sig> FIX XML */
+#else
 	    _("\nsig>%s%s</sig></log>\n"),          /* <sig> FIX XML */
+#endif
 	    log_msg.signature, log_msg.timestamp);
   else
     sprintf(log_msg.sig,                            /* known to fit  */
+#ifdef FIX_XML
+	    _("\n<sig>%s</sig></log>\n"),            /* <sig> FIX XML */
+#else
 	    _("\nsig>%s</sig></log>\n"),            /* <sig> FIX XML */
+#endif
 	    log_msg.signature);
 #ifdef SH_STEALTH
   /* don't encode the line breaks (0 + last char)
@@ -711,11 +779,13 @@ int  sh_log_file (char *errmsg)
   log_msg.msg[status] = '\0';
   sl_strlcat (log_msg.msg,   log_msg.sig, status + 2*KEY_LEN + 32);
 #ifdef SH_STEALTH
-  if (NULL != sl_strstr(log_msg.msg, N_("EXIT")))
+  if (NULL != sl_strstr(log_msg.msg, N_("EXIT")) &&
+      NULL == sl_strstr(log_msg.msg, N_("remote_host")))
     {
       sl_strlcat (log_msg.msg,  N_("</trail>"), status + 2*KEY_LEN + 32); 
 #else
-  if (NULL != sl_strstr(log_msg.msg,  _("msg=\"EXIT\"")))
+  if (NULL != sl_strstr(log_msg.msg,  _("msg=\"EXIT\"")) &&
+      NULL == sl_strstr(log_msg.msg,  _("remote_host")))
     {
       sl_strlcat (log_msg.msg,   _("</trail>"), status + 2*KEY_LEN + 32); 
 #endif
