@@ -52,6 +52,10 @@
 #include "sh_mail.h"
 #include "sh_forward.h"
 
+#if defined(WITH_DATABASE)
+#include "sh_tools.h"
+#endif
+
 #if defined(WITH_EXTERNAL)
 #include "sh_extern.h"
 #endif
@@ -106,7 +110,7 @@ int sh_error_verify (char * s)
 {
   char * foo;
 
-  if (s[0] == '/')  /* FIXME  a filename */
+  if (s[0] == '/')
     {
       foo = sh_tiger_hash_gpg (s, TIGER_FILE, 0);
       fprintf (stdout, _("%s\n"),  foo);
@@ -520,16 +524,26 @@ int sh_error_setexport(char *  str_s)
 
 /* set severity for printing
  */
+extern void dlog_set_active(int flag);
+
 int sh_error_setprint(char *  str_s)
 {
   static int reject = 0;
+  int        retval;
+
   if (reject == 1)
     return (0);
 
   if (sh.flag.opts == S_TRUE)   
     reject = 1;
 
-  return (sh_error_set_level(str_s, &errFlags.printlevel));
+  retval = sh_error_set_level(str_s, &errFlags.printlevel);
+
+  if (0 != (errFlags.printlevel & SH_ERR_INFO))
+    dlog_set_active(1);
+  if (0 != (errFlags.printlevel & SH_ERR_ALL))
+    dlog_set_active(2);
+  return retval;
 }
 
 
@@ -579,7 +593,7 @@ int sh_error_set_external (char * str_s)
 #endif
 
 #if defined(WITH_DATABASE)
-/* set severity for external
+/* set severity for database
  */
 int sh_error_set_database (char * str_s)
 {
@@ -636,6 +650,9 @@ void sh_error_handle (int sev, char * file, long line,
 #ifdef SH_WITH_CLIENT
   char   * ex_msg;
 #endif
+#if defined(WITH_DATABASE)
+  char   * escape_msg;
+#endif
 
   static int    own_block = 0;
 
@@ -834,7 +851,24 @@ void sh_error_handle (int sev, char * file, long line,
 	  if (database_block == 0)
 	    {
 	      database_block = 1;
-	      sh_database_insert (lmsg->msg);
+#ifndef SH_STANDALONE
+	      if (msg_id == MSG_TCP_MSG 
+#ifdef INET_SYSLOG
+		  || msg_id == MSG_INET_SYSLOG
+#endif
+		  )
+		{
+		  /* do not escape twice
+		   */
+		  sh_database_insert (lmsg->msg);
+		}
+	      else
+#endif
+		{
+		  escape_msg = sh_tools_safe_name(lmsg->msg, 0);
+		  sh_database_insert (escape_msg);
+		  SH_FREE(escape_msg);
+		}
 	      database_block = 0;
 	    }
 	}
