@@ -1138,6 +1138,8 @@ static void kill_sh_dirlist (struct sh_dirent * dirlist)
   return;
 }
   
+/* -- add an entry to a directory listing
+ */
 static struct sh_dirent * addto_sh_dirlist (struct dirent * thisEntry, 
 					    struct sh_dirent * dirlist)
 {
@@ -1163,6 +1165,8 @@ static struct sh_dirent * addto_sh_dirlist (struct dirent * thisEntry,
   return this;
 }
 
+/* -- check a single directory and its content
+ */
 int sh_files_checkdir (int iclass, int idepth, char * iname)
 {
   struct sh_dirent * dirlist = NULL;
@@ -1184,6 +1188,7 @@ int sh_files_checkdir (int iclass, int idepth, char * iname)
   int             class  = 0;
   int             rdepth_next;
   int             class_next;
+  int             file_class_next;
 
   int             checked_flag  = S_FALSE;
   int             cchecked_flag = S_FALSE;
@@ -1335,19 +1340,25 @@ int sh_files_checkdir (int iclass, int idepth, char * iname)
       sl_strlcat(tmpcat, "/",                   PATH_MAX+2);
     sl_strlcat(tmpcat, dirlist->sh_d_name,      PATH_MAX+2);
     
-    dst_ptr       = fileList;
-    rdepth_next   = rdepth - 1;
-    class_next    = class;
-    checked_flag  = -1;
-    cchecked_flag = -1;
+    dst_ptr         = fileList;
+    rdepth_next     = rdepth - 1;
+    class_next      = class;
+    file_class_next = class;
+    checked_flag    = -1;
+    cchecked_flag   = -1;
     
     while (dst_ptr) 
       {
 	if (0 == sl_strcmp(dst_ptr->name,  tmpcat))
 	  {
-	    class_next    = dst_ptr->class;
-	    checked_flag  = dst_ptr->checked;
-	    cchecked_flag = dst_ptr->childs_checked;
+	    /* Tue Aug  6 22:13:27 CEST 2002 introduce file_class_next
+	     * this fixes the problem that a policy for the directory
+	     * inode erroneously becomes a policy for the directory itself.
+	     */
+	    file_class_next    = dst_ptr->class;
+	    /* class_next         = dst_ptr->class; */
+	    checked_flag       = dst_ptr->checked;
+	    cchecked_flag      = dst_ptr->childs_checked;
 	    break;
 	  }
 	dst_ptr = dst_ptr->next;
@@ -1372,7 +1383,7 @@ int sh_files_checkdir (int iclass, int idepth, char * iname)
 	 */
 	if (dst_ptr && sh.flag.reportonce == S_TRUE)
 	  dummy = dst_ptr->reported;
-	checkit = sh_files_filecheck (class_next, 
+	checkit = sh_files_filecheck (file_class_next, 
 				      iname, 
 				      dirlist->sh_d_name,
 				      &dummy);
@@ -1566,8 +1577,10 @@ ShFileType sh_files_filecheck (int class, char * dirName, char * fileName,
    */
   if (theFile.c_mode[0] == '-')
     {
+      tmpname = sh_util_safe_name (fullpath); /* fixed in 1.5.4 */
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_FI_CSUM,
-		       fileHash, fullpath);
+		       fileHash, tmpname);
+      SH_FREE(tmpname);
     } 
       
   if      ( sh.flag.checkSum == SH_CHECK_INIT && sh.flag.update == S_FALSE ) 
