@@ -198,24 +198,6 @@ opttable_t op_table[] = {
     N_("Print usage information"),  
     HAS_ARG_NO, 
     sh_getopt_usage },
-#if defined(WITH_TPT) && !defined(WITH_TRACE)
-  { N_("tracelevel"),  
-    'z', 
-    N_("Set > 0 for debug"),  
-    HAS_ARG_YES, 
-    set_ebene },
-#elif defined(WITH_TRACE)
-  { N_("milestone"),  
-    'i', 
-    N_("Set milestone for trace/debug (all: -1)"),  
-    HAS_ARG_YES, 
-    set_stein },
-  { N_("tracelevel"),  
-    'z', 
-    N_("Set level for trace/debug"),  
-    HAS_ARG_YES, 
-    set_ebene },
-#endif
   /* last entry -- required !! -- */
   { NULL, 
     '\0',     
