@@ -668,6 +668,8 @@ int sh_gpg_check_sign (long file_1, long file_2, int what)
     {
       TPT(((0), FIL__, __LINE__, _("msg=<GPG_CHECK: FD1 = %d>\n"), fd1));
       TPT(((0), FIL__, __LINE__, _("msg=<GPG_CHECK: FD2 = %d>\n"), fd2));
+      dlog(1, FIL__, __LINE__, 
+	   _("This looks like an unexpected internal error.\n"));
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EXIT_ABORT1, sh.prg_name);
       aud_exit (FIL__, __LINE__, EXIT_FAILURE);
       SL_RETURN( (-1), _("sh_gpg_check_sign"));
@@ -712,6 +714,9 @@ int sh_gpg_check_sign (long file_1, long file_2, int what)
 	{
 	  /* fp mismatch
 	   */
+	  dlog(1, FIL__, __LINE__, 
+	       _("The fingerprint of the signing key: %s\ndoes not match the compiled-in fingerprint: %s.\nTherefore the signature could not be verified.\n"), 
+	       gp.conf_fp, SH_GPG_FP);
 	  sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_SUBGEN, 
 		      _("Fingerprint mismatch"), 
 		      _("gpg_check_sign"));
@@ -745,6 +750,8 @@ int sh_gpg_check_sign (long file_1, long file_2, int what)
 	{
 	  /* ID or fp not equal 
 	   */
+	  dlog(1, FIL__, __LINE__, 
+	       _("The fingerprint or ID of the signing key is not the same for the\nconfiguration file and the file signature database.\nTherefore the signature could not be verified.\n"));
 	  tmp  = sh_util_safe_name (gp.conf_id);
 	  tmp2 = sh_util_safe_name (gp.data_id);
 	  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_START_GH2,
@@ -758,10 +765,16 @@ int sh_gpg_check_sign (long file_1, long file_2, int what)
 	}
     }
 
+  if (status != SH_GPG_OK) 
+    {
+      dlog(1, FIL__, __LINE__, 
+	   _("The signature of the configuration file or the file signature database\ncould not be verified.Possible reasons are:\n - the signature key is not in the private keyring of UID %d,\n - there is no keyring in %s/.gnupg, or\n - the file is not signed - did you move /filename.asc to /filename ?\nTo create a signed file, use:\n   gpg -a --clearsign --not-dash-escaped FILE\n   mv FILE.asc FILE\n"), 
+	   (int) sl_ret_euid(), sh.effective.home);
+    }
 
   TPT(((0), FIL__, __LINE__, _("msg=<Status = %d>\n"), status));
 
-  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EXIT_ABORT1, sh.prg_name);
+  sh_error_handle((-1), FIL__, __LINE__, status, MSG_EXIT_ABORT1, sh.prg_name);
   aud_exit (FIL__, __LINE__, EXIT_FAILURE);
 
   return (-1); /* make compiler happy */
