@@ -145,7 +145,7 @@ static char * quote_string (char * str)
     {
       if (str[j] == '\n')
 	{
-	  tmp2 = int2hex((unsigned char) 'n');
+	  tmp2 = int2hex((unsigned char) '\n'); /* was 'n', fixed in 1.5.4 */
 	  tmp[k] = QUOTE_CHAR; ++k;
 	  tmp[k] = tmp2[0];    ++k;
 	  tmp[k] = tmp2[1];
@@ -227,7 +227,7 @@ static unsigned short *  swap_short (unsigned short * iptr)
 
 /* MAX_PATH_STORE must be >= KEY_LEN
  */
-#define MAX_PATH_STORE SH_BUFSIZE+1
+#define MAX_PATH_STORE 12287
 
 typedef struct store_info {
   /*
@@ -556,7 +556,7 @@ static int IsInit = 0;
  * Fast forward to start of data
  *
  ******************************************************************/
-int sh_hash_setdataent (SL_TICKET fd, char * line)
+int sh_hash_setdataent (SL_TICKET fd, char * line, int size)
 {
   long i;
 
@@ -566,10 +566,14 @@ int sh_hash_setdataent (SL_TICKET fd, char * line)
 
   while (1) 
     {
-      i =  sh_unix_getline (fd, line, SH_BUFSIZE);
+      i =  sh_unix_getline (fd, line, size-1);
       if (i < 0 ) 
 	{
 	  SH_FREE(line);
+	  dlog(1, FIL__, __LINE__, 
+	       _("The file signature database: %s does not\ncontain any data, or the start-of-file marker is missing (unlikely,\nunless modified by hand).\n"),
+	       (NULL == file_path('D', 'R'))? _("(null)"):file_path('D', 'R'));
+	       
 	  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_P_NODATA,
 			   ( (NULL == file_path('D', 'R')) ? _("(null)") :
 			     file_path('D', 'R'))
@@ -592,7 +596,7 @@ int sh_hash_setdataent (SL_TICKET fd, char * line)
  * Read next record
  *
  ******************************************************************/
-sh_file_t *  sh_hash_getdataent (SL_TICKET fd, char * line)
+sh_file_t *  sh_hash_getdataent (SL_TICKET fd, char * line, int size)
 {
   sh_file_t * p;
   sh_filestore_t ft;
@@ -637,18 +641,20 @@ sh_file_t *  sh_hash_getdataent (SL_TICKET fd, char * line)
 
   if (ft.mark != REC_MAGIC)
     {
-      /* fprintf(stderr, "FIXME ft.mark != REC_MAGIC\n"); */
       SH_FREE(p);
       SL_RETURN( NULL, _("sh_hash_getdataent"));
     }
 
   /* Read next record -- Part Two -- Fullpath
    */
-  i =  sh_unix_getline (fd, line, SH_BUFSIZE);
+  i =  sh_unix_getline (fd, line, size-1);
   if (i < 0 ) 
     {
       SH_FREE(line);
       SH_FREE(p);
+      dlog(1, FIL__, __LINE__, 
+	   _("There is a corrupt record in the file signature database: %s\nThe file path is missing.\n"),
+	   (NULL == file_path('D', 'R'))? _("(null)"):file_path('D', 'R'));
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_P_NODATA,
 			   ( (NULL == file_path('D', 'R')) ? _("(null)") :
 			     file_path('D', 'R'))
@@ -667,12 +673,15 @@ sh_file_t *  sh_hash_getdataent (SL_TICKET fd, char * line)
 
   /* Read next record -- Part Three -- Linkpath
    */
-  i =  sh_unix_getline (fd, line, SH_BUFSIZE);
+  i =  sh_unix_getline (fd, line, size-1);
   if (i < 0 ) 
     {
       SH_FREE(line);
       SH_FREE(fullpath);
       SH_FREE(p);
+      dlog(1, FIL__, __LINE__, 
+	   _("There is a corrupt record in the file signature database: %s\nThe link path (or its placeholder) is missing.\n"),
+	   (NULL == file_path('D', 'R'))? _("(null)"):file_path('D', 'R'));
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_P_NODATA,
 		       ( (NULL == file_path('D', 'R')) ? _("(null)") :
 			 file_path('D', 'R'))
@@ -765,8 +774,13 @@ void sh_hash_init ()
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_D_DSTART);
       fd = sh_forward_req_file(_("DATA"));
       if (SL_ISERROR(fd))
-	aud_exit (FIL__, __LINE__, EXIT_FAILURE);
-
+	{
+	  dlog(1, FIL__, __LINE__, 
+	       _("Could not retrieve the file signature database from the server.\nPossible reasons include:\n - the server is not running,\n - session key negotiation failed (see the manual for proper setup), or\n - the server cannot access the file.\n")); 
+	  sh_error_handle ((-1), FIL__, __LINE__, fd, MSG_EXIT_ABORT1, 
+			   sh.prg_name);
+	  aud_exit (FIL__, __LINE__, EXIT_FAILURE);
+	}
       sl_rewind (fd);
 
       tiger_fd = fd;
@@ -787,6 +801,9 @@ void sh_hash_init ()
 	  {
 	    TPT(( 0, FIL__, __LINE__, _("msg=<Error opening: %s>\n"), 
 		  file_path('D', 'R')));
+	    dlog(1, FIL__, __LINE__, 
+		 _("Could not open the local file signature database for reading because\nof the following error: %s\nIf this is a permission problem, you need to change file permissions\nto make the file readable for the effective UID: %d\n"), 
+		 sl_get_errmsg(), (int) sl_ret_euid());
 	    sh_error_handle ((-1), FIL__, __LINE__, fd, MSG_EXIT_ABORT1, 
 			     sh.prg_name);
 	    aud_exit (FIL__, __LINE__, EXIT_FAILURE);
@@ -801,6 +818,8 @@ void sh_hash_init ()
 			    KEY_LEN)
 	    && sh.flag.checkSum != SH_CHECK_INIT) 
 	  {
+	    dlog(1, FIL__, __LINE__, 
+		 _("The checksum of the file signature database has changed since startup.\n"));
 	    sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_E_AUTH,
 			     ( (NULL == file_path('D', 'R')) ? _("(null)") :
 			       file_path('D', 'R') )
@@ -819,8 +838,8 @@ void sh_hash_init ()
    */
   fdTmp = open_tmp();
 
-  buf = (char *) SH_ALLOC(4096);
-  while ( (bufc = sh_unix_getline (fd, buf, 4095)) > 0) 
+  buf = (char *) SH_ALLOC(8192);
+  while ( (bufc = sh_unix_getline (fd, buf, 8191)) > 0) 
     {
       if (flag_pgp == S_FALSE &&
 	  (0 == sl_strcmp(buf, _("-----BEGIN PGP SIGNED MESSAGE-----"))||
@@ -876,15 +895,15 @@ void sh_hash_init ()
 #endif
   /* } new 1.4.8 check sig also for files downloaded from server */
 
-  line = SH_ALLOC(SH_BUFSIZE+1);
+  line = SH_ALLOC(MAX_PATH_STORE+1);
 
   /* fast forward to start of data
    */
-  sh_hash_setdataent(fd, line);
+  sh_hash_setdataent(fd, line, MAX_PATH_STORE);
 
   while (1) 
     {
-      p = sh_hash_getdataent (fd, line);
+      p = sh_hash_getdataent (fd, line, MAX_PATH_STORE);
       if (p != NULL)
 	{
 	  hashinsert (p); 
@@ -938,12 +957,11 @@ void sh_hash_hashdelete ()
  ******************************************************************/ 
 void sh_hash_pushdata (file_type * buf, char * fileHash)
 {
-  /* sh_file_t * pp; */
   static long p_count = 0;
-  /* long        pi; */
-  char      * tmp;
 
-  /* unsigned short eo_m; */
+  char      * tmp;
+  size_t      tmp_len = 0;
+  size_t      old_len = 0;
 
   static SL_TICKET fd = -1;
   sh_filestore_t p;
@@ -952,13 +970,16 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
   struct stat sbuf;
   int    status = 0;
 
-  char fullpath[MAX_PATH_STORE+1];
-  char linkpath[MAX_PATH_STORE+1];
+  char *  fullpath = NULL;
+  char *  linkpath = NULL;
 
   char * line = NULL;
 
   SL_ENTER(_("sh_hash_pushdata"));
 
+  fullpath = SH_ALLOC(MAX_PATH_STORE+1);
+  linkpath = SH_ALLOC(MAX_PATH_STORE+1);
+
   linkpath[0] =  '-'; 
   linkpath[1] = '\0'; 
   fullpath[0] =  '-'; 
@@ -971,9 +992,11 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
   if (NULL == file_path('D', 'W') || 
       0 == sl_strcmp(file_path('D', 'W'), _("REQ_FROM_SERVER")))
     {
+      dlog(1, FIL__, __LINE__, 
+	   _("You need to configure a local path for initializing the database\nlike ./configure --with-data-file=REQ_FROM_SERVER/some/local/path\n"));
       sh_error_handle((-1), FIL__, __LINE__, 0, MSG_EXIT_ABORTS,
 		      sh.prg_name, 
-		      _("No local path for database initialisation"), 
+		      _("No local path for database specified"), 
 		      _("sh_hash_pushdata"));
       aud_exit(FIL__, __LINE__, EXIT_FAILURE);
     }
@@ -981,6 +1004,8 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
 
   if (isfirst == 1)  
     {
+      /* Warn that file already exists
+       */
       if (0 == retry_lstat(FIL__, __LINE__, file_path('D', 'W'), &sbuf))
 	sh_error_handle((-1), FIL__, __LINE__, 0, MSG_FI_DBEX,
 			file_path('D', 'W'));
@@ -990,20 +1015,31 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
   if (sh.flag.update == S_FALSE)
     {
       fd = -1;
-      if ( SL_ISERROR(fd = sl_open_write(file_path('D', 'W'), SL_YESPRIV)))
+      if ( SL_ISERROR(fd = sl_open_write(file_path('D', 'W'), SL_YESPRIV))) {
+	SH_FREE(fullpath);
+	SH_FREE(linkpath);
 	SL_RET0(_("sh_hash_pushdata"));
-      if ( SL_ISERROR(sl_forward(fd)))
+      }
+      if ( SL_ISERROR(sl_forward(fd))) {
+	SH_FREE(fullpath);
+	SH_FREE(linkpath);
 	SL_RET0(_("sh_hash_pushdata"));
+      }
     }
   else /* update == TRUE */
     {
       if (isfirst == 1)
 	{
-	  if ( SL_ISERROR(fd = sl_open_rdwr(file_path('D', 'W'), SL_YESPRIV)))
+	  if ( SL_ISERROR(fd = sl_open_rdwr(file_path('D', 'W'), SL_YESPRIV))){
+	    SH_FREE(fullpath);
+	    SH_FREE(linkpath);
 	    SL_RET0(_("sh_hash_pushdata"));
-	  line = SH_ALLOC(SH_BUFSIZE+1);
-	  if (SL_ISERROR(sh_hash_setdataent (fd, line)))
+	  }
+	  line = SH_ALLOC(MAX_PATH_STORE+1);
+	  if (SL_ISERROR(sh_hash_setdataent (fd, line, MAX_PATH_STORE)))
 	    {
+	      SH_FREE(fullpath);
+	      SH_FREE(linkpath);
 	      SH_FREE(line);
 	      SL_RET0(_("sh_hash_pushdata"));
 	    }
@@ -1011,22 +1047,34 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
 	}
     }
 	 
-  if (buf != NULL) {
+  if (buf != NULL && buf->fullpath != NULL) {
+
+    old_len = sl_strlen(buf->fullpath);
+#if defined(SH_STEALTH)
+    sh_do_encode(buf->fullpath, old_len);
+#endif
     tmp = quote_string(buf->fullpath);
-    if (sl_strlen(tmp) <= MAX_PATH_STORE) 
+    tmp_len = sl_strlen(tmp);
+#if defined(SH_STEALTH)
+    sh_do_decode(buf->fullpath, old_len);
+#endif
+
+    if (tmp_len <= MAX_PATH_STORE) 
       {
 	sl_strlcpy(fullpath, buf->fullpath, MAX_PATH_STORE+1);
+	/*
 	if (sl_strlen(buf->fullpath) < (MAX_PATH_STORE-3))
 	  {
 	    fullpath[MAX_PATH_STORE-2] = '\0';
 	    fullpath[MAX_PATH_STORE-1] = '\n';
 	  }
+	*/
       } 
     else 
       {
 	sl_strlcpy(fullpath, 
 		   sh_tiger_hash (buf->fullpath,
-				  TIGER_DATA,sl_strlen(buf->fullpath)), 
+				  TIGER_DATA, old_len), 
 		   KEY_LEN+1);
       }
     SH_FREE(tmp);
@@ -1037,13 +1085,23 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
 #endif
 
   tmp = quote_string(fullpath);
-  sl_strlcpy(fullpath, tmp, sl_strlen(tmp)+1); 
+  sl_strlcpy(fullpath, tmp, MAX_PATH_STORE+1);
   SH_FREE(tmp);
 
-  if (buf != NULL && buf->c_mode[0] == 'l') 
+  if (buf != NULL && buf->c_mode[0] == 'l' && buf->linkpath != NULL) 
     {  
+
+      old_len = sl_strlen(buf->linkpath);
+#if defined(SH_STEALTH)
+      sh_do_encode(buf->linkpath, old_len);
+#endif
       tmp = quote_string(buf->linkpath);
-      if (sl_strlen(tmp) <= MAX_PATH_STORE) 
+      tmp_len = sl_strlen(tmp);
+#if defined(SH_STEALTH)
+      sh_do_decode(buf->linkpath, old_len);
+#endif
+
+      if (tmp_len <= MAX_PATH_STORE) 
 	{
 	  sl_strlcpy(linkpath, buf->linkpath, MAX_PATH_STORE+1);  
 	} 
@@ -1051,7 +1109,7 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
 	{
 	  sl_strlcpy(linkpath, 
 		     sh_tiger_hash (buf->linkpath,
-				    TIGER_DATA,sl_strlen(buf->linkpath)),
+				    TIGER_DATA, old_len),
 		     KEY_LEN+1);
 	}
       SH_FREE(tmp);
@@ -1060,7 +1118,7 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
       sh_do_encode(linkpath, sl_strlen(linkpath));
 #endif
       tmp = quote_string(linkpath);
-      sl_strlcpy(linkpath, tmp, sl_strlen(tmp)+1); 
+      sl_strlcpy(linkpath, tmp, MAX_PATH_STORE+1);
       SH_FREE(tmp);
     }
 
@@ -1150,6 +1208,9 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
   if (sh.flag.update != S_TRUE)
     sl_close (fd);
 
+  SH_FREE(fullpath);
+  SH_FREE(linkpath);
+
   SL_RET0(_("sh_hash_pushdata"));
 }
 
@@ -2033,7 +2094,6 @@ int hash_remove_tree (char * s)
 	{
 	  if (sl_strncmp(s, p->fullpath, len) == 0)
 	    { 
-	      /* fprintf(stderr, "FIXME: remove %s\n", p->fullpath); */
 	      p->allignore  = S_TRUE;
 	    }
 	}
@@ -2080,16 +2140,16 @@ void sh_hash_list_db_entry_full_detail (sh_file_t * p)
   /* AIX has a completely different definition (%lli instead of "lli") 
    */
   if ('c' == p->theFile.c_mode[0] || 'b' == p->theFile.c_mode[0])
-    snprintf(str, 80, "PRIi64", p->theFile.rdev);
+    sl_snprintf(str, 80, "PRIi64", p->theFile.rdev);
   else
-    snprintf(str, 80, "PRIi64", p->theFile.size);
+    sl_snprintf(str, 80, "PRIi64", p->theFile.size);
 #else
   /* not AIX
    */
   if ('c' == p->theFile.c_mode[0] || 'b' == p->theFile.c_mode[0])
-    snprintf(str, 80, "%"PRIi64, p->theFile.rdev);
+    sl_snprintf(str, 80, "%"PRIi64, p->theFile.rdev);
   else
-    snprintf(str, 80, "%"PRIi64, p->theFile.size);
+    sl_snprintf(str, 80, "%"PRIi64, p->theFile.size);
 #endif
   printf( _(" %8s"), str);
 
@@ -2181,7 +2241,7 @@ int sh_hash_list_db (char * db_file)
       return -1; 
     }
 
-  line = SH_ALLOC(SH_BUFSIZE+1);
+  line = SH_ALLOC(MAX_PATH_STORE+1);
 
   if ( SL_ISERROR(fd = sl_open_read(db_file, SL_YESPRIV))) 
     {
@@ -2192,11 +2252,11 @@ int sh_hash_list_db (char * db_file)
 
   /* fast forward to start of data
    */
-  sh_hash_setdataent(fd, line);
+  sh_hash_setdataent(fd, line, MAX_PATH_STORE);
 
   while (1) 
     {
-      p = sh_hash_getdataent (fd, line);
+      p = sh_hash_getdataent (fd, line, MAX_PATH_STORE);
       if (p != NULL)
 	{
 	  if (ListFullDetail == S_FALSE)
@@ -2206,7 +2266,6 @@ int sh_hash_list_db (char * db_file)
 	}
       else
 	{
-	  /* fprintf(stderr, "FIXME: p == NULL\n"); */
 	  break;
 	}
     }
