@@ -31,6 +31,10 @@
  */
 int sh_hash_list_db (char * db_file);
 
+/* List database content with full detail
+ */
+int set_full_detail (char * c);
+
 /* Read the database from disk.
  */
 void sh_hash_init (void);
