@@ -928,6 +928,10 @@ static FILE * sh_mail_start_conn (int aFlag)
    */
   if (!sh_mail_wait (220, connFile)) 
     {
+      sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_NET, 
+		      _("Timeout on SMTP session init"), 
+		      _("sh_mail_start_conn"), 
+		      _("mail"), sh.host.name);
       TPT(( 0, FIL__, __LINE__, _("msg=<Timeout>\n")));
       MBLK( fclose(connFile); )
       SL_RETURN( NULL, _("sh_mail_start_conn"));
@@ -966,8 +970,11 @@ static FILE * sh_mail_start_conn (int aFlag)
   /* tell them who we are
    */
   sl_strlcpy (this_address, DEFAULT_SENDER, 256);
-  sl_strlcat (this_address, "@", 256);
-  sl_strlcat (this_address, sh.host.name, 256);
+  if (NULL == strchr(this_address, '@'))
+    {
+      sl_strlcat (this_address, "@", 256);
+      sl_strlcat (this_address, sh.host.name, 256);
+    }
 
   TPT(( 0, FIL__, __LINE__,  _("msg=<MAIL FROM:<%s>>%c%c"), 
 	this_address, 13, 10));
@@ -1136,6 +1143,8 @@ static int sh_mail_wait(int code, FILE * m_socket)
 {
   int rcode, g;
 
+  char errmsg[128];
+
   enum { 
     WAIT_CODE_START, 
     WAIT_CODE, 
@@ -1226,6 +1235,14 @@ static int sh_mail_wait(int code, FILE * m_socket)
 	   _("msg=<mail_wait: OK got %d (%d) need %d (%d)>\n"),
 	   rcode, (int)(rcode/100), code, (int)(code/100) ));
       g = ((int)(rcode/100) == (int)(code/100)) ? 1 : 0;
+      if (g != 1)
+	{
+	  sprintf(errmsg,                         /* known to fit  */
+		  _("Bad response (%d), expected %d"), rcode, code);
+	  sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_NET, 
+			  errmsg, _("sh_mail_wait"), 
+			  _("mail"), _("SMTP server"));
+	}
       waited_time = time(NULL) - waited_time;
       time_wait -= waited_time;
       TPT((0, FIL__, __LINE__, 
