@@ -205,16 +205,30 @@ int sh_readconf_read (void)
    */
   if (local_file == 1)
     {
-      if (0 != tf_trust_check (file_path('C', local_flag), SL_YESPRIV)
-	  ||
-	  SL_ISERROR(fd = sl_open_read(file_path('C',local_flag),SL_YESPRIV)))
+      if (0 != tf_trust_check (file_path('C', local_flag), SL_YESPRIV))
 	{
 	  sl_get_euid(&euid);
+	  dlog(1, FIL__, __LINE__, 
+	       _("The configuration file: %s is untrusted, i.e. an\nuntrusted user owns or can write to some directory in the path.\n"), 
+	       ( (NULL == file_path('C', local_flag)) 
+			     ? _("(null)") : file_path('C', local_flag) ));
+	  sh_error_handle ((-1), FIL__, __LINE__, EACCES, MSG_TRUST, 
+			   (long) euid, 
+			   ( (NULL == file_path('C', local_flag)) 
+			     ? _("(null)") : file_path('C', local_flag) )
+			   );
+	  aud_exit (FIL__, __LINE__, EXIT_FAILURE);
+	}
+      if (SL_ISERROR(fd = sl_open_read(file_path('C',local_flag),SL_YESPRIV)))
+	{
+	  sl_get_euid(&euid);
+	  dlog(1, FIL__, __LINE__, 
+	       _("Could not open the local file signature database for reading because\nof the following error: %s\nIf this is a permission problem, you need to change file permissions\nto make the file readable for the effective UID: %d\n"), 
+	       sl_get_errmsg(), (int) euid);
 	  sh_error_handle ((-1), FIL__, __LINE__, EACCES, MSG_NOACCESS, 
 			   (long) euid, 
 			   ( (NULL == file_path('C', local_flag)) 
 			     ? _("(null)") : file_path('C', local_flag) )
-			   
 			   );
 	  aud_exit (FIL__, __LINE__, EXIT_FAILURE);
 	}
@@ -299,6 +313,8 @@ int sh_readconf_read (void)
 	    sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_SUBGEN,
 			    _("second signed message in file"),
 			    _("sh_readconf_read"));
+	    dlog(1, FIL__, __LINE__, 
+		 _("There seems to be more than one signed message in the configuration\nfile. Please make sure there is only one signed message.\n"));
 	    sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_EXIT_ABORT1,
 			     sh.prg_name);
 	    aud_exit (FIL__, __LINE__,EXIT_FAILURE);
