@@ -109,6 +109,9 @@ char * errorExplain (int err_num)
 
 #endif
 
+/* ----------   old version, obsolete  -------------
+ */
+#if 0
 char * sh_tools_safe_name(const char * str, int flag)
 {
   register int  i = 0;
@@ -187,6 +190,187 @@ char * sh_tools_safe_name(const char * str, int flag)
   retval[i] = '\0';
   SL_RETURN( retval, _("sh_tools_safe_name"));
 }
+#else
+/* ----------   new version  -------------
+ */
+/* --- recode all \blah escapes to '=XX' format, and also code all
+ *     remaining unprintable chars                                 ---
+ */
+char * sh_tools_safe_name (const char * instr, int flag)
+{
+  unsigned char c;
+  const  char * p;
+  char   tmp[4];
+  char * outstr;
+  int    len;
+  int    i = 0;
+  unsigned char   val_octal = '\0';
+
+  SL_ENTER(_("sh_tools_safe_name"));
+
+  if (instr)
+    len = (3 * strlen(instr)) + 4;
+  else
+    len = 1;
+
+  outstr = SH_ALLOC(len);
+
+  outstr[0] = '\0';
+  tmp[3]    = '\0';
+
+  p = instr;
+
+  while (p && *p)
+    {
+      c = *p;
+
+      if (*p == '\n')
+	{
+	  outstr[i] = ' '; ++i; ++p;
+	  continue;
+	}
+
+#ifdef SH_USE_XML
+      if (flag == 1 && (*p) == '"')
+	{ 
+	  sprintf(&outstr[i], "=%02x", c);   i+=3; ++p;
+	  continue;
+	} 
+      else if (flag == 1 && (*p) == '&')
+	{ 
+	  sprintf(&outstr[i], "=%02x", c);   i+=3; ++p;
+	  continue;
+	} 
+      else if (flag == 1 && (*p) == '<') 
+	{     /* left angle       */
+	  sprintf(&outstr[i], "=%02x", c);   i+=3; ++p;
+	  continue;
+	} 
+      else if (flag == 1 && (*p) == '>') 
+	{     /* right angle      */
+	  sprintf(&outstr[i], "=%02x", c);   i+=3; ++p;
+	  continue;
+	} 
+#endif
+
+      if ( (*p) != '\\' && (*p) != '&' && (*p) != '='  && (*p) != '\'') 
+        {
+	  if (c < 31 || c > 126)
+	    {
+	      sprintf(&outstr[i], "=%02x", c);   i+=3; ++p;
+	    }
+	  else
+	    {
+	      outstr[i] = *p;
+	      ++i; ++p;
+	    }
+	  continue;
+	}
+      else if ((*p) == '\'')
+	{
+	  sprintf(&outstr[i], "=%02x", c);   i+=3; ++p;
+	}
+      else if (*p == '=')
+	{
+	  if (p[1] != '"')
+	    { sprintf(&outstr[i], "=%02x", c);   i+=3; ++p; }
+	  else
+	    { outstr[i] = *p; ++i; ++p; }
+	}
+      else if (*p == '\\')
+	{
+	  ++p;
+	  if (!p)
+	    break;
+	  if (!(*p))
+	    break;
+
+	  c = *p;
+
+	  switch (*p) {
+	  case '\\':
+	    sprintf(&outstr[i], "=%02x", c);   i+=3; ++p;
+	    break;
+	  case 'n':
+	    sprintf(&outstr[i], "=%02x", '\n'); i+=3; ++p;
+	    break;
+	  case 'b':
+	    sprintf(&outstr[i], "=%02x", '\b'); i+=3; ++p;
+	    break;		       
+	  case 'r':		       
+	    sprintf(&outstr[i], "=%02x", '\r'); i+=3; ++p;
+	    break;		       
+	  case 't':		       
+	    sprintf(&outstr[i], "=%02x", '\t'); i+=3; ++p;
+	    break;		       
+	  case 'v':		       
+	    sprintf(&outstr[i], "=%02x", '\v'); i+=3; ++p;
+	    break;		       
+	  case 'f':		       
+	    sprintf(&outstr[i], "=%02x", '\f'); i+=3; ++p;
+	    break;		       
+	  case '\'':		       
+	    sprintf(&outstr[i], "=%02x", '\''); i+=3; ++p;
+	    break;		       
+	  case '\"':	/* also encode quoted '"' */ 	       
+	    sprintf(&outstr[i], "=%02x", '\"'); i+=3; ++p;
+	    break;		       
+	  case ' ':		       
+	    sprintf(&outstr[i], "=%02x", ' ');  i+=3; ++p;
+	    break;
+	  default:
+	    if (strlen(p) < 3)
+	      {
+		p += strlen(p);
+	      }
+	    else
+	      {
+		tmp[0] = p[0]; tmp[1] = p[1]; tmp[2] = p[2]; 
+		val_octal = (unsigned char) strtoul(tmp, (char **)NULL, 8);
+		if (val_octal != '\0') { 
+		  c = val_octal;
+		  sprintf(&outstr[i], "=%02x", c); i+=3;
+		} 
+		p += 3;
+	      }
+	  }
+	}
+      else /* *p == '&' */
+	{
+	  ++p;
+	  if (!p || !(*p)) 
+	    break;
+
+	  if (p[0] == 'a' && p[1] == 'm' && p[2] == 'p' && p[3] == ';')
+	    {
+	      sprintf(&outstr[i], "=%02x", '&');  i+=3; p += 4;
+	    }
+	  else if (p[0] == 'q' && p[1] == 'u' && p[2] == 'o' && p[3] == 't' &&
+		   p[4] == ';')
+	    {
+	      sprintf(&outstr[i], "=%02x", '"');  i+=3; p += 5;
+	    }
+	  else if (p[0] == 'l' && p[1] == 't' && p[2] == ';')
+	    {
+	      sprintf(&outstr[i], "=%02x", '<');  i+=3; p += 3;
+	    }
+	  else if (p[0] == 'g' && p[1] == 't' && p[2] == ';')
+	    {
+	      sprintf(&outstr[i], "=%02x", '>');  i+=3; p += 3;
+	    }
+	  else /* conserve the '&' */
+	    {
+	      outstr[i] = '&'; ++i;
+	    }
+	}     
+    } /* while (p && *p) */
+  
+  outstr[i] = '\0';
+  flag = 0; /* fix compiler warning */
+  SL_RETURN( outstr, _("sh_tools_safe_name"));
+}
+
+#endif
 
 /* extern int h_errno; */ 
 
@@ -1042,6 +1226,9 @@ SL_TICKET open_tmp ()
 
   if (0 !=  tf_trust_check (my_tmp_dir, SL_YESPRIV))
       {
+	dlog(1, FIL__, __LINE__, 
+	     _("The directory for temporary files: %s is untrusted, i.e. an\nuntrusted user owns or can write to some directory in the path.\n"), 
+	     my_tmp_dir);
 	sh_error_handle ((-1), FIL__, __LINE__, EACCES, MSG_TRUST,
 			 (long) sh.effective.uid,
 			 my_tmp_dir);
