@@ -369,7 +369,7 @@ void sh_unix_sigexit_fast (int mysignal
   strncpy (sh_sig_msg, sh_unix_siglist(mysignal), 40);
 #endif
 
-#if !defined(WITH_TRACE) && defined(SL_DEBUG)
+#if defined(SL_DEBUG)
   sl_stack_print();
 #endif
 
@@ -386,7 +386,7 @@ void sh_unix_sigexit_fast (int mysignal
 #endif
 
 #if !defined(SL_DEBUG)
-  _exit(EXIT_FAILURE);
+  _exit(mysignal);
 #else
   raise(SIGFPE);
 #endif
@@ -625,6 +625,9 @@ int sh_unix_self_hash (char * c)
 		     _("000000000000000000000000000000000000000000000000")
 		     ))
     {
+      dlog(1, FIL__, __LINE__, 
+	   _("Could not checksum my own executable because of the\nfollowing error: %s\nIf this is a permission problem, you need to change file permissions\nto make the file readable for the effective UID: %d\n"), 
+	   sl_get_errmsg(), (int) sl_ret_euid());
       sh_error_handle ((-1), FIL__, __LINE__, EACCES, MSG_NOACCESS,
 		       (long) sh.real.uid, c);
       aud_exit (FIL__, __LINE__, EXIT_FAILURE);
@@ -643,9 +646,13 @@ int sh_unix_self_check ()
 		      KEY_LEN))
     SL_RETURN((0), _("sh_unix_self_check"));
 
-  sh_error_handle ((-1), FIL__, __LINE__, EACCES, MSG_E_AUTH,
-		   sh.exec.path);
-  SL_RETURN((-1), _("sh_unix_self_check"));
+ 
+    dlog(1, FIL__, __LINE__, 
+	 _("The checksum of the executable: %s has changed since startup.\n"),
+	 sh.exec.path);
+    sh_error_handle ((-1), FIL__, __LINE__, EACCES, MSG_E_AUTH,
+		     sh.exec.path);
+    SL_RETURN((-1), _("sh_unix_self_check"));
 }
 
 
@@ -681,7 +688,6 @@ static int tf_add_trusted_user_int(char * c)
  succe:
   count = sl_trust_add_user(pwid);
   SL_RETURN((count), _("tf_add_trusted_user_int"));
-  
 }
 
 int tf_add_trusted_user(char * c)
@@ -712,6 +718,9 @@ int tf_add_trusted_user(char * c)
   SL_RETURN((0), _("tf_add_trusted_user"));
 }
 
+extern long   sl_trust_baduid();
+extern long   sl_trust_badgid();
+
 int tf_trust_check (char * file, int mode)
 {
   char * tmp;
@@ -756,7 +765,51 @@ int tf_trust_check (char * file, int mode)
       if (status == SL_EBADUID   || status == SL_EBADGID || 
 	  status == SL_EBADOTH   || status == SL_ETRUNC  || 
 	  status == SL_EINTERNAL )
-	SL_RETURN((-1), _("tf_trust_check"));
+	{
+	  switch (status) {
+	  case SL_EINTERNAL:
+	    dlog(1, FIL__, __LINE__, 
+		 _("An internal error occured in the trustfile function.\n"));
+	    break;
+	  case SL_ETRUNC:
+	    tmp  = sh_util_safe_name (file);
+	    dlog(1, FIL__, __LINE__, 
+		 _("A filename truncation occured in the trustfile function.\nProbably the normalized filename for %s\nis too long. This may be due e.g. to deep or circular softlinks.\n"), 
+		 tmp);
+	    SH_FREE(tmp);
+	    break;
+	  case SL_EBADOTH:
+	    tmp  = sh_util_safe_name (file);
+	    p    = sl_trust_errfile();
+	    dlog(1, FIL__, __LINE__, 
+		 _("The path element: %s\nin the filename: %s is world writeable.\n"),
+		 p, tmp);
+	    SH_FREE(p);
+	    SH_FREE(tmp);
+	    break;
+	  case SL_EBADUID:
+	    tmp  = sh_util_safe_name (file);
+	    p    = sl_trust_errfile();
+	    dlog(1, FIL__, __LINE__, 
+		 _("The owner (UID = %ld) of the path element: %s\nin the filename: %s\nis not in the list of trusted users.\nTo fix the problem, you can:\n - run ./configure again with the option --with-trusted=0,...,UID\n   where UID is the UID of the untrusted user, or\n - use the option TrustedUser=UID in the configuration file.\n"),
+		 sl_trust_baduid(), p, tmp);
+	    SH_FREE(p);
+	    SH_FREE(tmp);
+	  case SL_EBADGID:
+	    tmp  = sh_util_safe_name (file);
+	    p    = sl_trust_errfile();
+	    dlog(1, FIL__, __LINE__, 
+		 _("The path element: %s\nin the filename: %s\nis group writeable (GID = %ld), and at least one of the group\nmembers (UID = %ld) is not in the list of trusted users.\nTo fix the problem, you can:\n - run ./configure again with the option --with-trusted=0,...,UID\n   where UID is the UID of the untrusted user, or\n - use the option TrustedUser=UID in the configuration file.\n"),
+		 p, tmp, sl_trust_badgid(), sl_trust_baduid());
+	    SH_FREE(p);
+	    SH_FREE(tmp);
+	    break;
+	  default:
+	    break;
+	  }
+	    
+	  SL_RETURN((-1), _("tf_trust_check"));
+	}
     }
 
   SL_RETURN((0), _("tf_trust_check"));
@@ -2721,6 +2774,8 @@ int sh_unix_getline_stealth (SL_TICKET fd, char * str, int len)
       off_data = first_hex_block(fd, &max_data);
       if (off_data == 0)
 	{
+	  dlog(1, FIL__, __LINE__, 
+	       _("The stealth config file does not contain any steganographically\nhidden data. This file must be an image file in _uncompressed_\npostscript format.\nTo hide data in it, use:\n   samhain_stealth -s postscript_file orig_config_file\n   mv postscript_file /path/to/config/file\n"));
 	  sh_error_handle ((-1), FIL__, __LINE__,  EIO, MSG_P_NODATA,
 			   _("Stealth config file."));
 	  aud_exit (FIL__, __LINE__, EXIT_FAILURE);
