@@ -3,6 +3,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
 
 #include <unistd.h>
@@ -19,6 +20,18 @@
 
 #undef BREAKEXIT
 #if defined(SCREW_IT_UP) && defined(__linux__) && defined(__i386__)
+
+#ifdef SH_DEBUG
+#define BREAKEXIT(expr) \
+  do { \
+    int ixi; \
+    for (ixi = 0; ixi < 8; ++ixi) { \
+      if ((*(volatile unsigned *)((unsigned) expr + ixi) & 0xff) == 0xcc)  \
+        { dlog(0, FIL__, __LINE__, _("BREAKEXIT")); _exit(EXIT_FAILURE); } \
+      } \
+    } \
+  while (1 == 0)
+#else
 #define BREAKEXIT(expr) \
   do { \
     int ixi; \
@@ -28,6 +41,8 @@
       } \
     } \
   while (1 == 0)
+#endif
+
 #else
 #define BREAKEXIT(expr)
 #endif
@@ -38,155 +53,81 @@
  *
  ****************************************************************/
 
+extern int sh_log_console (char * msg);
 
-#if defined(WITH_TRACE) || defined(WITH_TPT)
+static int dlogActive = 0;
 
-#define ALLE_STEINE -1
-
-#if !defined(WITH_TRACE)
-int stein_set = ALLE_STEINE;
-#else
-int stein_set = -99;
-#endif
-
-int stein     =  0;
-int ebene     =  0;
-int mindest   = -1;
-
-static int use_console = 0;
-void dbg_use_console()
+/* this is called from sh_error_setprint()
+ */
+void dlog_set_active(int flag)
 {
-  use_console = 1;
-  return;
+  dlogActive = flag;
 }
 
-extern char *  sh_log_console_name ();
-
-
-void dbg(int mod_ebene, char * file, int line, char * fmt, ...)
+/* flag = 0 debug messages
+ *      = 1 descriptive error messages
+ *      = 3 backtrace
+ */
+int dlog (int flag, char * file, int line,  const char *fmt, ...)
 {
-  FILE * dfile;
-  va_list ap;
-  char indent_it[32];
-  int  i, imax;
+  va_list     ap;
+  char        val[81];
+  char        msg[512];
+  char        tmp[512];
+  int         retval = 0;
 
-  if (ebene < 1) 
-    ++stein;
+#ifdef SH_STEALTH
+  /* 
+   * do not even print descriptive failure messages in stealth mode
+   */
+  if (dlogActive == 0)
+    return 0;
+  if (dlogActive == 1 && flag == 0) /* debug requires debug level */
+    return 0;
+#else
+  if (dlogActive == 0 && flag == 0) /* debug requires debug level */
+    return 0;
+#endif
 
-  if (mod_ebene > 0)
-    ebene += mod_ebene;
+  if (flag == 1)
+    {
+      sprintf        (val, _("\n---------  %10s "), file);
+      sl_strlcpy     (msg,    val,   80);
+      sprintf        (val, _(" --- %6d ---------\n"), line);
+      sl_strlcat     (msg,     val,   80);
+      sh_log_console (msg);
+    }
 
-  if (ebene < 31)
-    imax = ebene;
+  va_start (ap, fmt);
+  sl_strlcpy(tmp, fmt, 256);
+  retval = sl_strlen(tmp);
+  if (retval > 0 && tmp[retval-1] == '\n')
+    tmp[retval-1] = '\0';
+  retval = 0;
+  if (flag == 1)
+    sl_vsnprintf (msg, 255, tmp, ap);
   else
-    imax = 31;
-
-  /*
-  fprintf (stderr, _("DBG: file=<%s>, line=<%d> %d %d \n"), 
-		 file, line, stein_set, mindest);
-  */
-
-  for (i = 0; i < imax; ++i)
-    indent_it[i] = ' ';
-  indent_it[imax] = '\0';
-
-  if ( (stein == stein_set || stein_set == ALLE_STEINE ) && ebene <= mindest)
     {
-      if (ebene < 1)
-#if !defined(WITH_TRACE)
-	{
-	  if (0 == use_console)
-	    {
-	      fprintf (stderr, _("DBG: file=<%s>, line=<%d>, "), 
-		       file, line);
-	      va_start(ap, fmt);
-	      vfprintf  (stderr, fmt, ap);
-	      va_end(ap);
-	    }
-	  else
-	    {
-	      dfile = fopen(sh_log_console_name(), "w");
-	      fprintf (dfile, _("DBG: file=<%s>, line=<%d>, "), file, line);
-	      va_start(ap, fmt);
-	      vfprintf  (dfile, fmt, ap);
-	      va_end(ap);
-	      fclose (dfile);
-	    }
-	}
-#else
-      {
-	if (0 == use_console)
-	  fprintf (stderr, 
-		   _("MILEST: file=<%s>, line=<%d>, milestone=<%d>, "), 
-		   file, line, stein);
-	else
-	  {
-	    dfile = fopen(sh_log_console_name(), "w");
-	    fprintf (dfile, 
-		     _("MILEST: file=<%s>, line=<%d>, milestone=<%d>, "), 
-		     file, line, stein);
-	    va_start(ap, fmt);
-	    vfprintf  (dfile, fmt, ap);
-	    va_end(ap);
-	    fclose (dfile);
-	  }
-      }
-#endif
-      else
-	{
-	  if (0 == use_console)
-	    {
-	      fprintf (stderr, _("%sTRACE : file=<%s>, line=<%d>, "), 
-		       indent_it, file, line);
-	      va_start(ap, fmt);
-	      vfprintf  (stderr, fmt, ap);
-	      va_end(ap);
-	    }
-	  else
-	    {
-	      dfile = fopen(sh_log_console_name(), "w");
-	      fprintf (dfile, 
-		       _("%sTRACE : file=<%s>, line=<%d>, "),
-		       indent_it, file, line);
-	      va_start(ap, fmt);
-	      vfprintf  (dfile, fmt, ap);
-	      va_end(ap);
-	      fclose (dfile);
-	    }
-	}
+      sl_strlcpy   (msg,    file,   10);
+      sprintf      (val, _(" %d "), line);
+      sl_strlcat   (msg,     val,   16);
+      sl_vsnprintf (&msg[sl_strlen(msg)], 255, tmp, ap);
     }
+  va_end (ap);
+  retval = sh_log_console (msg);
 
-  if (mod_ebene < 0)
-    ebene += mod_ebene;
-
-  return;
+  if (flag == 1)
+    sh_log_console (_("\n-------------------------------------------\n"));
+  return retval;
 }
 
-int set_stein (char * c)
+extern char aud_err_message[64];
+char * sl_get_errmsg()
 {
-  stein_set = atoi(c);
-  if (stein_set < -1)
-    {
-      stein_set = -99;
-      return (-1);
-    }
-  return 0;
+  return &aud_err_message[0];
 }
-int set_ebene (char * c)
-{
-  mindest = atoi(c);
-  if (mindest < 0)
-    {
-      mindest = -99;
-      return (-1);
-    }
-  return 0;
-}
-/* WITH_TRACE */
-#endif
 
-
-#if !defined(WITH_TRACE) && defined(SL_DEBUG)
+#if defined(SL_DEBUG)
 #define SL_MAX_MYSTACK 128
 
 static char sl_mystack[SL_MAX_MYSTACK][32];
@@ -223,18 +164,23 @@ void sl_stack_pop()
 void sl_stack_print()
 {
   int  i;
-  FILE * dfile;
+  /* FILE * dfile; */
 
   if (sl_mystack_count > 0)
     {
+      /*
       dfile = fopen(_("samhain_backtrace"), "a");
-      fprintf (dfile, _("\nBacktrace:\n"));
+      fprintf (dfile, _("\nBacktrace:\n"));      
 	  
       for (i = 0; i < sl_mystack_count; ++i)
 	{
 	  fprintf(dfile, _("#%03d %s\n"), i, sl_mystack[i]);
 	}
       fclose (dfile);
+      */
+      dlog(3, FIL__, __LINE__, _("\nBacktrace:\n"));
+      for (i = 0; i < sl_mystack_count; ++i)
+	dlog(3, FIL__, __LINE__, _("#%03d %s\n"), i, sl_mystack[i]);
     } 
   return;
 }
@@ -320,9 +266,9 @@ extern int vsnprintf ( char *str, size_t n,
 
 #if !defined(HAVE_VSNPRINTF) || defined(HAVE_BROKEN_VSNPRINTF)
 static
-int sl_printf_count (const char * fmt, va_list  vl)
+size_t sl_printf_count (const char * fmt, va_list  vl)
 {
-  int  length       = 1;
+  size_t  length       = 1;
   int  fini         = 0;
   int  islong       = 0;
   int  islongdouble = 0;
@@ -387,7 +333,7 @@ int sl_printf_count (const char * fmt, va_list  vl)
 	    (void) va_arg (vl, long);
 	  else
 	    (void) va_arg (vl, int);
-	  islong == 0;
+	  islong = 0;
 	  length = length + 24;
 	  ++fmt;
 	  fini = 1;
@@ -479,7 +425,7 @@ int sl_printf_count (const char * fmt, va_list  vl)
 int sl_vsnprintf(char *str, size_t n,
 		 const char *format, va_list vl )
 {
-  int           total;
+  size_t         total;
 #if !defined(HAVE_VSNPRINTF) || defined(HAVE_BROKEN_VSNPRINTF)
   va_list       vl2;
 #endif
@@ -518,7 +464,7 @@ int sl_vsnprintf(char *str, size_t n,
 int sl_snprintf(char *str, size_t n,
 		const char *format, ... )
 {
-  int           total = 0;
+  size_t          total = 0;
   va_list       vl;
 #if !defined(HAVE_VSNPRINTF) || defined(HAVE_BROKEN_VSNPRINTF)
   va_list       vl2;
@@ -531,7 +477,7 @@ int sl_snprintf(char *str, size_t n,
   va_start (vl, format);
 #if defined(HAVE_VSNPRINTF) && !defined(HAVE_BROKEN_VSNPRINTF)
   vsnprintf (str, n, format, vl);
-  total = sl_strlen(str);
+  total = (size_t) sl_strlen(str);
 #else
   VA_COPY (vl2, vl);                   /* save the argument list           */
   total = sl_printf_count (format, vl);
@@ -858,6 +804,12 @@ int sl_get_euid(uid_t * ret)
   SL_IRETURN (SL_ENONE, _("sl_get_euid"));
 }
 
+uid_t sl_ret_euid()
+{
+  SL_REQUIRE(uids_are_stored == SL_TRUE, _("uids_are_stored == SL_TRUE"));
+  return (euid);
+}
+
 /*
  * This function returns the saved egid.
  * It calls abort() if the uid's are not saved already.
@@ -1097,7 +1049,7 @@ int sl_policy_get_root()
 /* 
  * Define a policy: Get real (irrevocably).
  * This function drops SUID privileges irrevocably.
- * Do nothing if not SUID.
+ * Do nothing if not SUID (? not true - drops if root).
  */
 int sl_policy_get_real(char * user)
 {
@@ -1110,7 +1062,6 @@ int sl_policy_get_real(char * user)
   if (euid == 0 || ruid == 0)
     {
       tempres = getpwnam(user);
-      fprintf(stderr, "user = %s", user);
       SL_REQUIRE (NULL != tempres, _("tempres != NULL"));
   
       rgid_orig = tempres->pw_gid;
@@ -1728,8 +1679,10 @@ int sl_read (SL_TICKET ticket, void * buf_in, size_t count)
 
  
   if (byteread == (-1))
-    SL_IRETURN(SL_EREAD, _("sl_read"));
-
+    {
+      TPT(( 0, FIL__, __LINE__, _("msg=<read error> errno=<%d>\n"), errno));
+      SL_IRETURN(SL_EREAD, _("sl_read"));
+    }
    SL_IRETURN(bytes, _("sl_read"));
 }
 
@@ -1861,6 +1814,18 @@ char * sl_trust_errfile()
   return &tf_path[0];
 }
 
+extern long tf_baduid;
+long   sl_trust_baduid()
+{
+  return tf_baduid;
+}
+
+extern long tf_badgid;
+long   sl_trust_badgid()
+{
+  return tf_badgid;
+}
+
 
 static int trust_count = 0;
 
