@@ -107,7 +107,7 @@
 #define retry_lstat(a,b,c,d) lstat(c,d)
 #define _(string)  string 
 #define N_(string) string
-#define MAXFILENAME     2048
+#define MAXFILENAME     4096
 static int sl_errno = 0;
 #define SL_ENONE         0
 #define SL_ENULL     -1024     /* Invalid use of NULL pointer.         */
@@ -199,6 +199,8 @@ int EUIDSLOT = sizeof(test_rootonly)/sizeof(long);
 int ORIG_EUIDSLOT = sizeof(test_rootonly)/sizeof(long);
 
 char tf_path[MAXFILENAME];		/* error path for trust function */
+long tf_baduid;
+long tf_badgid;
 
 static 
 int dirz(char *path)
@@ -504,6 +506,7 @@ int onlytrustedingrp(long grp, long *ulist)
 	  fprintf (stderr, 
 		   "trustfile: user=%s (gid %ld), not a trusted user --> ERROR\n", *p, grp);
 #endif 
+	  tf_baduid = w->pw_uid;
 	  SL_IRETURN(SL_FALSE, _("onlytrustedingrp"));
 	}
     }
@@ -551,6 +554,7 @@ int onlytrustedingrp(long grp, long *ulist)
 #ifdef TRUST_DEBUG
 	      fprintf(stderr,"trustfile: group member %s not found in trusted users --> ERROR\n", w->pw_name); 
 #endif
+	      tf_baduid = w->pw_uid;
 	      SL_IRETURN(SL_FALSE, _("onlytrustedingrp"));
 	    }
 	}
@@ -803,6 +807,7 @@ int sl_trustfile(char *fname, long *okusers, long *badusers)
 	  fprintf(stderr, "---------------------------------------------\n");
 #endif 
 	  (void) strcpy(tf_path, fexp);                  /* known to fit  */
+	  tf_baduid = (long) stbuf.st_uid;
 	  SL_IRETURN(SL_EBADUID, _("sl_trustfile"));
 	}
 
@@ -839,6 +844,7 @@ int sl_trustfile(char *fname, long *okusers, long *badusers)
 	  fprintf(stderr, "---------------------------------------------\n");
 #endif 
 	  (void) strcpy(tf_path, fexp);                  /* known to fit  */
+	  tf_badgid = (long) stbuf.st_gid;
 	  SL_IRETURN(SL_EBADGID, _("sl_trustfile"));
 	}
       /*
